//.b 	
//	Copyright 2001 [MEAN].BOT
//	
//	File: MA_time.h
//	
//	[MEAN] 
//
//.e	

//.b skip over if header has already been included.
#ifndef defMA_time_h

	// Prevent the reprocessing of this definition file by defining 
	// the alias used in the above "ifndef" check.
	#define defMA_time_h

	// Include all general definition files.
	#include "MB_Includes.h"
	#include <time.h>
	#include <limits.h>

//.begin ------------------------------------------
// Class tclCPUClock Definitions

	#define defDouble double

	#ifdef __linux__
		#define __int64 long long int
	#endif


	typedef long tinClock_lng;      // standard long integer;
	typedef __int64 tinClock_dbl;   // integer double precision OR equivalent if not available;
	typedef defDouble tflClock_dbl; // floating point double precision;

   // maximum resolution is in milliseconds
	// 1 millisecond = 10^-3 or 1/1000 of a second;
	const tinClock_lng cinMSecInSec   = 1000;
	const tflClock_dbl cflMSecInSec   = cinMSecInSec;

	// maximum ticks before clock() rollover;
	const tinClock_dbl cinMaxTickCount_Clock = LONG_MAX; // maxium signed long integer; 
	const tflClock_dbl cflMaxTickCount_Clock = cinMaxTickCount_Clock; 

	// maximum accumulated ticks allowed based on largest MS value that can
	// be stored in a signed long integer;
	// for a 32 bit long integer, the maximum amount will ensure a fatal rollover 
	// is not possible until about 65 years of continuous operation;
	const tinClock_dbl cinMaxTickCount_Total  = cinMaxTickCount_Clock * cinMSecInSec; // 2,147,483,647,000;
	const tinClock_lng cinMinTickCount_Total  = 0; 
	const tflClock_dbl cflMaxTickCount_Total  = cinMaxTickCount_Total;
	const tflClock_dbl cflMinTickCount_Total  = cinMinTickCount_Total;

   // total number of making up a full rollover cycle;
	const tinClock_dbl cinTicksInRollOver = cinMaxTickCount_Clock + 1; 
	const tflClock_dbl cflTicksInRollOver = cinTicksInRollOver; 

	const tinClock_lng cinMinsInHr    = 60;
	const tflClock_dbl cflMinsInHr	 = cinMinsInHr;
	const tinClock_lng cinSecsInMin	 = 60;
	const tflClock_dbl cflSecsInMin	 = cinSecsInMin;
	const tinClock_lng cinSecsInHr	 = cinSecsInMin * cinMinsInHr;
	const tflClock_dbl cflSecsInHr	 = cinSecsInHr;
	const tinClock_lng cinTicksInSec	 = CLOCKS_PER_SEC;
	const tflClock_dbl cflTicksInSec	 = cinTicksInSec;
	const tinClock_lng cinTicksInMSec = cinTicksInSec / cinMSecInSec;
	const tflClock_dbl cflTicksInMSec = cflTicksInSec / cflMSecInSec;
	const tinClock_lng cinTicksInMin	 = cinTicksInSec * cinSecsInMin;
	const tflClock_dbl cflTicksInMin	 = cflTicksInSec * cflSecsInMin;
	const tinClock_lng cinTicksInHr	 = cinTicksInSec * cinSecsInHr;
	const tflClock_dbl cflTicksInHr	 = cflTicksInSec * cflSecsInHr;

	const tinClock_lng cinMaxTimerInTicks = cinMaxTickCount_Clock;
	const tflClock_dbl cflMaxTimerInTicks = cinMaxTimerInTicks;
	const tinClock_lng cinMinTimerInTicks = 0;
	const tflClock_dbl cflMinTimerInTicks = cinMinTimerInTicks;
	const tinClock_lng cinMaxTimerInMSecs = cinMaxTimerInTicks / cinTicksInMSec;
	const tflClock_dbl cflMaxTimerInMSecs = cflMaxTimerInTicks / cflTicksInMSec;
	const tinClock_lng cinMinTimerInMSecs = 0;
	const tflClock_dbl cflMinTimerInMSecs = cinMinTimerInMSecs;
	const tinClock_lng cinMaxTimerInSecs  = cinMaxTimerInTicks / cinTicksInSec;
	const tflClock_dbl cflMaxTimerInSecs  = cflMaxTimerInTicks / cflTicksInSec;
	const tinClock_lng cinMinTimerInSecs  = 0;
	const tflClock_dbl cflMinTimerInSecs  = cinMinTimerInSecs;
	const tinClock_lng cinMaxTimerInMins  = cinMaxTimerInTicks / cinTicksInMin;
	const tflClock_dbl cflMaxTimerInMins  = cflMaxTimerInTicks / cflTicksInMin;
	const tinClock_lng cinMinTimerInMins  = 0;
	const tflClock_dbl cflMinTimerInMins  = cinMinTimerInMins;
	const tinClock_lng cinMaxTimerInHrs   = cinMaxTimerInTicks / cinTicksInHr;
	const tflClock_dbl cflMaxTimerInHrs   = cflMaxTimerInTicks / cflTicksInHr;
	const tinClock_lng cinMinTimerInHrs   = 0;
	const tflClock_dbl cflMinTimerInHrs   = cinMinTimerInHrs;

	class tclCPUClock
	{
		// accuracy = 1 millisecond or 1000th of a second per clock tick;
		private:
			tinClock_lng vinClock_Cur;
			tinClock_lng vinClock_Prev;
			tinClock_dbl vinClock_RunningTotal;
			tinClock_dbl vinRollOver_RunningTotal;

		public:
			tclCPUClock( );
			tinClock_dbl finGetClockInTicks( ); 
			tflClock_dbl fflGetClockInTicks( ){ return defDouble( finGetClockInTicks( ) ); }; 
			tinClock_lng finGetClockInMSecs( ){ return finGetClockInTicks( ) / cinTicksInMSec; };  
			tflClock_dbl fflGetClockInMSecs( ){ return fflGetClockInTicks( ) / cflTicksInMSec; };  
			tinClock_lng finGetClockInSecs( ){ return finGetClockInTicks( ) / cinTicksInSec; };  
			tinClock_lng finGetClock( ){ return finGetClockInSecs( ); };  
			tflClock_dbl fflGetClockInSecs( ){ return fflGetClockInTicks( ) / cflTicksInSec; };  
			tflClock_dbl fflGetClock( ){ return fflGetClockInSecs( ); };  
			tinClock_lng finGetClockInMins( ){ return finGetClockInTicks( ) / cinTicksInMin; }; 
			tflClock_dbl fflGetClockInMins( ){ return fflGetClockInTicks( ) / cflTicksInMin; }; 
			tinClock_lng finGetClockInHrs( ){ return finGetClockInTicks( ) / cinTicksInHr; }; 
			tflClock_dbl fflGetClockInHrs( ){ return fflGetClockInTicks( ) / cflTicksInHr; }; 

	}; // tclCPUClock

	typedef tclCPUClock* tcl_ptrCPUClock;

// Class tclCPUClock Definitions
//.end ------------------------------------------

	#define defTimerType double

	class tclTimer
	{	
		private:
			defTimerType vflTime_Start; // time clock started
			defTimerType vflTime_Stop;  // time out point

		public:
			tclTimer( void );
			void  fvoStop( void );
			void  fvoStart( defTimerType pflSeconds );
			void  fvoStartAddTime( defTimerType pflSeconds );
			defTimerType fflTimeLeft( void ); 
			defTimerType fflTimeElapsed( void ); 
			defTimerType fflTimerSetting( void ); 
			defTimerType fflGetStopTime( void ) { return vflTime_Stop; }; 
			defTimerType fflGetStartTime( void ) { return vflTime_Start; }; 
			bool  fboTimedOut( void );

	}; // tclTimer

#endif // defMA_time_h
